@@ -32,6 +32,8 @@ LOAD_BALANCER_ROWS = (
     'vip_port_id',
     'vip_qos_policy_id',
     'vip_subnet_id',
+    'tags',
+    'additional_vips',
 )
 
 LOAD_BALANCER_COLUMNS = (
@@ -79,7 +81,8 @@ LISTENER_ROWS = (
     'allowed_cidrs',
     'tls_ciphers',
     'tls_versions',
-    'alpn_protocols')
+    'alpn_protocols',
+    'tags')
 
 LISTENER_COLUMNS = (
     'id',
@@ -112,7 +115,9 @@ POOL_ROWS = (
     'crl_container_ref',
     'tls_enabled',
     'tls_ciphers',
-    'tls_versions')
+    'tls_versions',
+    'tags',
+    'alpn_protocols')
 
 POOL_COLUMNS = (
     'id',
@@ -138,8 +143,8 @@ MEMBER_ROWS = (
     'weight',
     'monitor_port',
     'monitor_address',
-    'backup'
-)
+    'backup',
+    'tags')
 
 MEMBER_COLUMNS = (
     'id',
@@ -168,7 +173,8 @@ L7POLICY_ROWS = (
     'id',
     'operating_status',
     'name',
-    'redirect_http_code')
+    'redirect_http_code',
+    'tags')
 
 L7POLICY_COLUMNS = (
     'id',
@@ -191,7 +197,8 @@ L7RULE_ROWS = (
     'project_id',
     'type',
     'id',
-    'operating_status')
+    'operating_status',
+    'tags')
 
 L7RULE_COLUMNS = (
     'id',
@@ -223,8 +230,8 @@ MONITOR_ROWS = (
     'id',
     'operating_status',
     'http_version',
-    'domain_name'
-)
+    'domain_name',
+    'tags')
 
 MONITOR_COLUMNS = (
     'id',
