@@ -12,6 +12,9 @@
 #   under the License.
 #
 
+import functools
+import ipaddress
+
 import munch
 from openstackclient.identity import common as identity_common
 from osc_lib import exceptions as osc_exc
@@ -171,6 +174,47 @@ def get_resource_id(resource, resource_name, name):
         raise osc_exc.CommandError(msg) from e
 
 
+def add_tags_attr_map(attr_map):
+    tags_attr_map = {
+        'tags': ('tags', list),
+        'any_tags': ('tags-any', list),
+        'not_tags': ('not-tags', list),
+        'not_any_tags': ('not-tags-any', list),
+    }
+    attr_map.update(tags_attr_map)
+
+
+def validate_vip_dict(vip_dict, client_manager):
+    # We have validation in two places -- _map_attrs checks sub-resources, and
+    # later _check_attrs does further api-specific validation. We need both for
+    # additional vips, so we may as well just do both here while we're at it.
+    if 'subnet_id' not in vip_dict:
+        raise osc_exc.CommandError(
+            'Additional VIPs must include a subnet-id.')
+    subnet_id = get_resource_id(client_manager.neutronclient.list_subnets,
+                                'subnets', vip_dict['subnet_id'])
+    vip_dict['subnet_id'] = subnet_id
+    if 'ip_address' in vip_dict:
+        try:
+            ipaddress.ip_address(vip_dict['ip_address'])
+        except ValueError as e:
+            raise osc_exc.CommandError(str(e))
+
+
+def handle_additional_vips(vips, client_manager):
+    additional_vips = []
+    for vip in vips:
+        vip_dict = {}
+        parts = vip.split(',')
+        for part in parts:
+            k, v = part.split('=')
+            vip_dict[k.replace('-', '_')] = v
+        validate_vip_dict(vip_dict, client_manager)
+        additional_vips.append(vip_dict)
+
+    return additional_vips
+
+
 def get_loadbalancer_attrs(client_manager, parsed_args):
     attr_map = {
         'name': ('name', str),
@@ -221,8 +265,13 @@ def get_loadbalancer_attrs(client_manager, parsed_args):
             client_manager.load_balancer.flavor_list
         ),
         'availability_zone': ('availability_zone', str),
-
+        'additional_vip': (
+            'additional_vips',
+            functools.partial(
+                handle_additional_vips, client_manager=client_manager)
+        ),
     }
+    add_tags_attr_map(attr_map)
 
     _attrs = vars(parsed_args)
     attrs = _map_attrs(_attrs, attr_map)
@@ -276,6 +325,7 @@ def get_listener_attrs(client_manager, parsed_args):
         'tls_versions': ('tls_versions', list),
         'alpn_protocols': ('alpn_protocols', list),
     }
+    add_tags_attr_map(attr_map)
 
     _attrs = vars(parsed_args)
     attrs = _map_attrs(_attrs, attr_map)
@@ -323,7 +373,9 @@ def get_pool_attrs(client_manager, parsed_args):
         'disable_tls': ('tls_enabled', lambda x: False),
         'tls_ciphers': ('tls_ciphers', str),
         'tls_versions': ('tls_versions', list),
+        'alpn_protocols': ('alpn_protocols', list),
     }
+    add_tags_attr_map(attr_map)
 
     _attrs = vars(parsed_args)
     attrs = _map_attrs(_attrs, attr_map)
@@ -365,6 +417,7 @@ def get_member_attrs(client_manager, parsed_args):
         'enable': ('admin_state_up', lambda x: True),
         'disable': ('admin_state_up', lambda x: False),
     }
+    add_tags_attr_map(attr_map)
 
     _attrs = vars(parsed_args)
     attrs = _map_attrs(_attrs, attr_map)
@@ -404,6 +457,7 @@ def get_l7policy_attrs(client_manager, parsed_args):
         'enable': ('admin_state_up', lambda x: True),
         'disable': ('admin_state_up', lambda x: False)
     }
+    add_tags_attr_map(attr_map)
 
     _attrs = vars(parsed_args)
     attrs = _map_attrs(_attrs, attr_map)
@@ -438,6 +492,7 @@ def get_l7rule_attrs(client_manager, parsed_args):
         'enable': ('admin_state_up', lambda x: True),
         'disable': ('admin_state_up', lambda x: False)
     }
+    add_tags_attr_map(attr_map)
 
     _attrs = vars(parsed_args)
     attrs = _map_attrs(_attrs, attr_map)
@@ -476,6 +531,7 @@ def get_health_monitor_attrs(client_manager, parsed_args):
         'http_version': ('http_version', float),
         'domain_name': ('domain_name', str)
     }
+    add_tags_attr_map(attr_map)
 
     _attrs = vars(parsed_args)
     attrs = _map_attrs(_attrs, attr_map)
@@ -651,8 +707,13 @@ def _format_str_if_need_treat_unset(data):
 
 
 def get_unsets(parsed_args):
-    return {arg: None for arg, value in vars(parsed_args).items() if
-            value is True and arg != 'wait'}
+    unsets = {}
+    for arg, value in vars(parsed_args).items():
+        if value and arg == 'tags':
+            unsets[arg] = value
+        elif value is True and arg not in ('wait', 'all_tag'):
+            unsets[arg] = None
+    return unsets
 
 
 def wait_for_active(status_f, res_id):
@@ -689,3 +750,23 @@ def wait_for_delete(status_f, res_id,
     except exceptions.OctaviaClientException as e:
         if e.code != 404:
             raise
+
+
+def set_tags_for_set(resource_get, resource_id, attrs, clear_tags=False):
+    if attrs.get('tags'):
+        resource = resource_get(resource_id)
+        tags = set([] if clear_tags else resource['tags'])
+        tags |= set(attrs['tags'])
+        attrs['tags'] = list(tags)
+    elif clear_tags:
+        attrs['tags'] = []
+
+
+def set_tags_for_unset(resource_get, resource_id, attrs, clear_tags=False):
+    if clear_tags:
+        attrs['tags'] = []
+    elif attrs.get('tags'):
+        resource = resource_get(resource_id)
+        tags = set(resource['tags'])
+        tags -= set(attrs['tags'])
+        attrs['tags'] = list(tags)
